﻿Type=Class
Version=2.2
B4J=true
@EndOfDesignText@
'WebSocket class

Sub Class_Globals

	Private WS As WebSocket
	
	Private lblSelectGoogleChartType As UILabel
	Private cboSelectGoogleChartType As UICombobox
	
	Private GooglePieChart As UIGoogleChart
	Private GoogleLineChart As UIGoogleChart
	Private GoogleGauge As UIGoogleChart
	Private GoogleGeoChart As UIGoogleChart
	
	Private myTimer As Timer

End Sub

Public Sub Initialize
	
End Sub

Private Sub WebSocket_Connected(WebSocket1 As WebSocket)

	WS = WebSocket1

	lblSelectGoogleChartType.Initialize(WS, "lblSelectGoogleChartType")
	lblSelectGoogleChartType.Resize(-1, 12, -1, -1)
	lblSelectGoogleChartType.SetText("Select a chart type:")
	
	cboSelectGoogleChartType.Initialize(WS, "cboSelectGoogleChartType")
	cboSelectGoogleChartType.Resize(130, -1, -1, -1)
	cboSelectGoogleChartType.AddItem("PieChart")
	cboSelectGoogleChartType.AddItem("LineChart")
	cboSelectGoogleChartType.AddItem("Gauge")
	cboSelectGoogleChartType.AddItem("GeoChart")

	DrawPieChart

End Sub

Private Sub cboSelectGoogleChartType_Change(Params As Map)

	If GooglePieChart.IsInitialized Then GooglePieChart.Remove
	If GoogleLineChart.IsInitialized Then GoogleLineChart.Remove
	If GoogleGauge.IsInitialized Then GoogleGauge.Remove
	If GoogleGeoChart.IsInitialized Then GoogleGeoChart.Remove

	myTimer.Enabled = False

	Dim SelectedValue As Future
	
	SelectedValue = cboSelectGoogleChartType.GetText
	
	Select Case SelectedValue.Value
	
		Case "PieChart"
			DrawPieChart
			
		Case "LineChart"
			DrawLineChart
			
		Case "Gauge"
			DrawGauge
			
		Case "GeoChart"
			DrawGeoChart
			
	End Select

End Sub	

Private Sub DrawPieChart()
	
	' #################
	' # PieChart
	' #################

	' https://google-developers.appspot.com/chart/interactive/docs/gallery/piechart

	GooglePieChart.Initialize(WS, "GooglePieChart", "PieChart")
	GooglePieChart.Resize(100, 100, 500, 300)
	GooglePieChart.SetCss("border", "1px solid black")
	
	' Set options
	
	Dim Options As Map = CreateMap("title":"How Much Pizza I Ate Last Night", _
																 "width":"500", _
																 "height":"300", _
																 "pieHole":"0.4")
												
	GooglePieChart.SetOptions(Options)
	
	' Set data
	
	Dim Data As List
	Data.Initialize()
	Data.Add(Array As Object("Topping",   "Slices"))
	Data.Add(Array As Object("Mushrooms",        3))
	Data.Add(Array As Object("Onions",           1))
	Data.Add(Array As Object("Olives",           1))
	Data.Add(Array As Object("Zucchini",         1))
	Data.Add(Array As Object("Pepperoni",        2))
	
	GooglePieChart.SetData(Data)
	GooglePieChart.DrawChart

End Sub

Sub DrawLineChart()

	' #################
	' # LineChart
	' #################

	' https://google-developers.appspot.com/chart/interactive/docs/gallery/linechart

	GoogleLineChart.Initialize(WS, "GoogleLineChart", "LineChart")
	GoogleLineChart.Resize(100, 100, 500, 300)
	GoogleLineChart.SetCss("border", "1px solid black")

	' Set options
	
	Dim Options As Map = CreateMap("title":"Company Performance", "width":"500", "height":"300")
	GoogleLineChart.SetOptions(Options)
	
	' Set data
	
	Dim Data As List
	Data.Initialize()
	Data.Add(Array As Object("Year", "Sales", "Expenses"))
	Data.Add(Array As Object("2004",    1000,        400))
	Data.Add(Array As Object("2005",    1170,        460))
	Data.Add(Array As Object("2006",     660,       1120))
	Data.Add(Array As Object("2007",    1030,        540))
	
	GoogleLineChart.SetData(Data)
	GoogleLineChart.DrawChart

End Sub

Sub DrawGauge()

	' #################
	' # Gauge
	' #################

	' https://google-developers.appspot.com/chart/interactive/docs/gallery/gauge

	GoogleGauge.Initialize(WS, "GoogleGauge", "Gauge")
	GoogleGauge.Resize(100, 100, 500, 300)
	GoogleGauge.SetCss("border", "1px solid black")
	
	' Set options

	Dim Options As Map = CreateMap("width":"500", _
																 "height":"300", _
																 "redFrom":"90", _
																 "redTo":"100", _
																 "yellowFrom":"75", _
																 "yellowTo":"90", _
																 "minorTicks":"5")
	
	GoogleGauge.SetOptions(Options)
	
	' Set data
	
	Dim Data As List
	Data.Initialize()
	Data.Add(Array As Object("Label", "Value"))
	Data.Add(Array As Object("Memory",     80))
	Data.Add(Array As Object("CPU",        55))
	Data.Add(Array As Object("Network",    68))
	
	GoogleGauge.SetData(Data)
	GoogleGauge.DrawChart

	' setting a timer for updating Gauge-Values

	myTimer.Initialize("myTimer", 1000)
	myTimer.Enabled = True

End Sub

Sub DrawGeoChart()

	' #################
	' # GeoChart
	' #################

	' https://google-developers.appspot.com/chart/interactive/docs/gallery/geochart

	GoogleGeoChart.Initialize(WS, "GoogleGeoChart", "GeoChart")
	GoogleGeoChart.Resize(100, 100, 500, 300)
	GoogleGeoChart.SetCss("border", "1px solid black")
	
	Dim Options As Map = CreateMap()
	GoogleGeoChart.SetOptions(Options)
	
	' Set data
	
	Dim Data As List
	Data.Initialize()
	Data.Add(Array As Object("Country", "Popularity"))
	Data.Add(Array As Object("Germany",          200))
	Data.Add(Array As Object("United States",    300))
	Data.Add(Array As Object("Brazil",           400))
	Data.Add(Array As Object("Canada",           500))
	Data.Add(Array As Object("France",           600))
	Data.Add(Array As Object("RU",               700))
	
	GoogleGeoChart.SetData(Data)
	GoogleGeoChart.DrawChart

End Sub

Private Sub myTimer_Tick

	Dim Data As List
	Data.Initialize()
	
	Data.Add(Array As Object("Label",       "Value"))
	Data.Add(Array As Object("Memory",  Rnd(0, 100)))
	Data.Add(Array As Object("CPU",     Rnd(0, 100)))
	Data.Add(Array As Object("Network", Rnd(0, 100)))
	
	GoogleGauge.SetData(Data)
	GoogleGauge.DrawChart
	
	WS.Flush
	
End Sub

Private Sub WebSocket_Disconnected

End Sub